/*****************************************************************************

Copyright 2001-2050 Saulo Popov Zambiasi. All rights reserved.
Last modified: 2006/07/30 17:35:00

Borg
	Classe responsavel por dividir a tarefa em partes, criando um processo
	para cada um dos servidores requisitados.

	Na definicao dos servidores e portas para conexoes podem existir dois
	servidores em uma mesma maquina. Nesse caso, é necessário especificar
	portas diferentes para cada um deles.
   
	Neste exercicio eh apresentado um exemplo de criacao de tres processos
	para 3 daemons. Observa-se que os daemons precisam estar rodando nos
	servidores especificados caso não seja localhost.
   
Exemplo de servidores e portas:

	String[] servidores = { "192.168.1.1", "192.168.1.2", "192.168.1.2" }
	int[] portas        = {  4410,          4411 ,         4412         }

******************************************************************************/

public class Borg {
	static public void main (String[] arguments) {
		String[] servidores = { "localhost", "localhost", "localhost"};
		int[]    portas     = {  4410,        4411,        4412 };
		int      processos = servidores.length;
		int      numero = 0;

		// Pega o numero passado como parametro para o calculo do fatorial
		if (arguments.length == 1) {
			numero = Integer.parseInt(arguments[0]);
		} else {
			System.out.println("Utilize: java numero");
			return;
		}
		// Divide o numero em partes, cada parte para cada processo
		int bloco = (int) (numero / processos);
		int contador = 1;
		
		// Criacao dos processos para controlar a comunicacao com os daemons
		BorgProcess[] ap = new BorgProcess[processos];
		for (int i = 0; i < processos; i++) {
			try {
				int inicio = contador;
				int fim = contador + bloco;
				if (fim > numero) fim = numero;
				contador = fim + 1;
				String mensagem = "fatorial " + inicio + " " + fim;
				// Cria o processo e envia a mensagem para o mesmo
				ap[i] = new BorgProcess(servidores[i], portas[i], mensagem);
			} catch (Exception e) {
				System.out.println("ERRO: " + e.toString());
			}
		}

		// Espera o retorno de cada processo e multiplica os resultados.
		long resultado = 1;
		boolean terminou = false;
		while (!terminou) {
			terminou = true;
			for (int i = 0; i < processos; i++){
				if (!ap[i].terminado) {
					terminou = false;
				} else if (ap[i].temMensagem) {
					resultado *= Long.parseLong(ap[i].buscaMensagem());
				}
			}
		}
		// Imprime o resultado na tela.
    	System.out.println("FATORIAL(" + numero+  ") = " + resultado);
	}
}
