/*****************************************************************************

Copyright 2001-2050 Saulo Popov Zambiasi. All rights reserved.
Last modified: 2006/07/30 17:35:00

Borg Process
   Esta classe  responsavel por cada um dos processos que vai conectar-se
   com cada um dos daemons responsaveis pelas operacoes. A classe conecta-se,
   envia uma mensagem e espera um retorno.

******************************************************************************/

import java.io.*;
import java.net.*;

public class BorgProcess implements Runnable {
	String servidor = "";			// IP ou nome do servidor
	String mensagem = "";			// Mensagem
	boolean temMensagem = false;	// Mensagem recebida
	int porta = 4413;				// Porta da conexao
	boolean terminado = false;		// Trabalho terminado
	private Thread runner;

	// Constructor, recebendo como parametro o ip ou nome
	// do servidor e a mensagem que sera enviada.
	BorgProcess(String srv, int p, String msg) {
		mensagem = msg;
		servidor = srv;
		if (p != 0) porta = p;
		if (runner == null) {
			runner = new Thread(this);
			runner.start();
		}
	}

	// Retorna a mensagem recebida, se ela existir.
	String buscaMensagem() {
		temMensagem = false;
		return mensagem;
	}

	// Execucao do processo
	public void run() {
		try {
			// Conexao com o daemon
			Socket conexao = new Socket(servidor, porta);
			conexao.setSoTimeout(20000);
			
			// Envia a mensagem para o daemon
			System.out.println("ENVIA PARA [" + servidor + 
				":" + porta + "]: " + mensagem);
			BufferedOutputStream bos = new BufferedOutputStream(
				conexao.getOutputStream());
			PrintWriter os = new PrintWriter(bos, false);
			os.println(mensagem);
			os.flush();

			// recebe o resultado do daemon em forma de uma mensagem
			BufferedReader in = new BufferedReader(
				new InputStreamReader(conexao.getInputStream()));
			mensagem = in.readLine();
			System.out.println("RECEBE DE  [" + servidor +
				":" + porta + "]: " + mensagem);
			temMensagem = true;
			conexao.close();
		} catch (IOException e) {
			System.out.println("Erro: " + e.getMessage());
		}
		terminado = true;
	}
}
